==> guiloader-2.10.0.tar.bz2: guiloader-2.10.0/guiloader/containers.c <==
/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "containers.h"

// CrowChild -----------------------------------------------------------------

GType crow_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowChild), 
			0, (GInstanceInitFunc)crow_child_init,
		};
		object_type= g_type_register_static(G_TYPE_OBJECT, "CrowChild", &object_info, 0);
	}
	return object_type;
}

void crow_child_init(CrowChild * child) {
	child->widget= NULL;
	child->place.a= 0;
	child->place.b= 0;
	child->place.c= 0;
	child->place.d= 0;
}

// CrowRowChild --------------------------------------------------------------

GType crow_row_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowRowChild), 
			0, NULL,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowRowChild", &object_info, 0);
	}
	return object_type;
}

// CrowXYChild ---------------------------------------------------------------

GType crow_xy_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowXYChild), 
			0, NULL,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowXYChild", &object_info, 0);
	}
	return object_type;
}

// CrowBoxChild --------------------------------------------------------------

GType crow_box_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowBoxChild), 
			0, (GInstanceInitFunc)crow_box_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowBoxChild", &object_info, 0);
	}
	return object_type;
}

void crow_box_child_init(CrowBoxChild * child) {
	CROW_CHILD(child)->place.b= GTK_PACK_START;
	child->options= CROW_PACK_EXPAND_WIDGET;
	child->padding= 0;
}

// CrowButtonBoxChild --------------------------------------------------------

GType crow_button_box_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowButtonBoxChild), 
			0, (GInstanceInitFunc)crow_button_box_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_BOX_CHILD, "CrowButtonBoxChild", &object_info, 0);
	}
	return object_type;
}

void crow_button_box_child_init(CrowButtonBoxChild * child) {
	child->secondary= FALSE;
}

// CrowPanedChild ------------------------------------------------------------

GType crow_paned_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowPanedChild), 
			0, (GInstanceInitFunc)crow_paned_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowPanedChild", &object_info, 0);
	}
	return object_type;
}

void crow_paned_child_init(CrowPanedChild * child) {
	child->resize= TRUE;
	child->shrink= TRUE;
}

// CrowNotebookChild ---------------------------------------------------------

GType crow_notebook_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowNotebookChild), 
			0, (GInstanceInitFunc)crow_notebook_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowNotebookChild", &object_info, 0);
	}
	return object_type;
}

void crow_notebook_child_init(CrowNotebookChild * child) {
	child->tabLabel= NULL;
	child->tabLabelWidget= NULL;
	child->menuLabel= NULL;
	child->menuLabelWidget= NULL;
	child->packOptions= CROW_PACK_SHRINK;
	child->detachable= FALSE;
	child->reorderable= FALSE;
}

// CrowTableChild ------------------------------------------------------------

GType crow_table_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowTableChild), 
			0, (GInstanceInitFunc)crow_table_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowTableChild", &object_info, 0);
	}
	return object_type;
}

void crow_table_child_init(CrowTableChild * child) {
	CROW_CHILD(child)->place.c= 1;
	CROW_CHILD(child)->place.d= 1;
	child->paddingX= 0;
	child->paddingY= 0;
	child->xExpand= TRUE;
	child->xFill= TRUE;
	child->xShrink= FALSE;
	child->yExpand= TRUE;
	child->yFill= TRUE;
	child->yShrink= FALSE;
}

// -----------------------------------------------------------------------------

gboolean ContainerIsEmpty(GtkContainer * container) {
	return gtk_container_get_children(container) == NULL;
}

gboolean GetExpand(CrowPackOptions packOptions) {
	return packOptions != CROW_PACK_SHRINK;
}

gboolean GetFill(CrowPackOptions packOptions) {
	return packOptions == CROW_PACK_EXPAND_WIDGET;
}

GtkAttachOptions GetAttachOptions(gboolean expand, gboolean fill, gboolean shrink) {
	GtkAttachOptions result= 0;
	if(expand)
		result |= GTK_EXPAND;
	if(fill)
		result |= GTK_FILL;
	if(shrink)
		result |= GTK_SHRINK;
	return result;
}

GtkWidget * CreatePlaceholder(void) {
	GtkWidget * widget= gtk_drawing_area_new();
	gtk_widget_set_size_request(widget, 40, 40);
	g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(ExposePlaceholder), NULL);
	gtk_widget_show(widget);
	return widget;
}

gboolean ExposePlaceholder(GtkWidget * widget, GdkEventExpose * event, gpointer data) {
	DrawGrid(widget->window, widget->style, FALSE);
	return FALSE;
}

void ContainerFixedSet(GtkFixed * fixed, PtrArray * children) {
	int i;
	for(i= 0; i < children->size; ++i) {
		CrowChild * child= CROW_CHILD(children->ptrs[i]);
		gtk_fixed_put(fixed, child->widget, child->place.a, child->place.b);
	}
}

void ContainerLayoutSet(GtkLayout * layout, PtrArray * children) {
	int i;
	for(i= 0; i < children->size; ++i) {
		CrowChild * child= CROW_CHILD(children->ptrs[i]);
		gtk_layout_put(layout, child->widget, child->place.a, child->place.b);
	}
}

void ContainerBinSet(GtkBin * bin, PtrArray * children) {
	CrowChild * child;
	Check(children && children->size==1);
	child= CROW_CHILD(children->ptrs[0]);
	gtk_container_add(GTK_CONTAINER(bin), child->widget);
}

void ContainerScrolledWindowSet(GtkScrolledWindow * scroll, PtrArray * children) {
	CrowChild * child;
	Check(children && children->size==1);
	child= CROW_CHILD(children->ptrs[0]);
	if(GTK_WIDGET_GET_CLASS(child->widget)->set_scroll_adjustments_signal != 0)
		gtk_container_add(GTK_CONTAINER(scroll), child->widget);
	else {
		GtkViewport * viewport;
		gtk_scrolled_window_add_with_viewport(scroll, child->widget);
		viewport= GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(scroll)));
		g_object_set_data(G_OBJECT(viewport), "CrowAutoViewportHint", "true");
	}
}

void ContainerDialogSet(GtkDialog * dialog, PtrArray * children) {
	CrowChild * child;
	GtkBox * box;
	Check(children && children->size==1);
	child= CROW_CHILD(children->ptrs[0]);
	box= GTK_BOX(dialog->vbox); 
	gtk_box_pack_start_defaults(box, child->widget);
}

void ContainerPanedSet(GtkPaned * paned, PtrArray * children) {
	int i;
	Check(children && children->size <= 2);
	for(i= 0; i < children->size; ++i) {
		CrowPanedChild * child= CROW_PANED_CHILD(children->ptrs[i]);
		if(CROW_CHILD(child)->place.a==0)
			gtk_paned_pack1(paned, CROW_CHILD(child)->widget, child->resize, child->shrink);
		else {
			Check(CROW_CHILD(child)->place.a==1);
			gtk_paned_pack2(paned, CROW_CHILD(child)->widget, child->resize, child->shrink);
		}
	}
}

void ContainerBoxSet(GtkBox * box, PtrArray * children, int capS, int capE) {
	PCrowChild * sorted= g_slice_alloc0(sizeof(gpointer)*(capS+capE));

	int i;
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowChild * child= CROW_CHILD(children->ptrs[i]);
			if(child->place.b == GTK_PACK_START) {
				Check(child->place.a < capS);
				Check(!sorted[child->place.a]);
				sorted[child->place.a]= child;
			} else {
				Check(child->place.b == GTK_PACK_END);
				Check(child->place.a < capE);
				Check(!sorted[capS + child->place.a]);
				sorted[capS + child->place.a]= child;
			}
		}

	for(i=0; i<capS+capE; ++i) {
		CrowChild * child= sorted[i];
		CrowBoxChild * boxChild;
		
		if(!child) {
			child= g_object_new(CROW_TYPE_BOX_CHILD, NULL);
			if(i<capS) {
				child->place.a= i;
				child->place.b= GTK_PACK_START;
			} else {
				child->place.a= i-capS;
				child->place.b= GTK_PACK_END;
			}
			child->widget= CreatePlaceholder();
		}

		boxChild= CROW_BOX_CHILD(child);
		if(child->place.b == GTK_PACK_START)
			gtk_box_pack_start(box, child->widget, GetExpand(boxChild->options), GetFill(boxChild->options), boxChild->padding);
		else
			gtk_box_pack_end(box, child->widget, GetExpand(boxChild->options), GetFill(boxChild->options), boxChild->padding);

		if(!sorted[i])
			g_object_unref(child);
		
	}
	
	g_slice_free1(sizeof(gpointer)*(capS+capE), sorted);	
}

void ContainerButtonBoxSet(GtkButtonBox * box, PtrArray * children, int capS, int capE) {
	int i;
	ContainerBoxSet(GTK_BOX(box), children, capS, capE);
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowButtonBoxChild * child= CROW_BUTTON_BOX_CHILD(children->ptrs[i]);
			if(child->secondary)
				gtk_button_box_set_child_secondary(box, CROW_CHILD(child)->widget, TRUE);
		}
}

void ContainerNotebookSet(GtkNotebook * notebook, PtrArray * children, int capacity) {
	PCrowChild * sorted= g_slice_alloc0(sizeof(gpointer)*capacity);

	int i;
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowChild * child= CROW_CHILD(children->ptrs[i]);
			Check(child->place.a < capacity);
			Check(!sorted[child->place.a]);
			sorted[child->place.a]= child;
		}

	for(i=0; i<capacity; ++i) {
		CrowChild * child= sorted[i];
		CrowNotebookChild * nbChild;
		
		if(!child) {
			child= g_object_new(CROW_TYPE_NOTEBOOK_CHILD, NULL);
			child->place.a= i;
			child->widget= CreatePlaceholder();
		}

		nbChild= CROW_NOTEBOOK_CHILD(child);
		gtk_notebook_append_page(notebook, child->widget, nbChild->tabLabelWidget);
		if(!nbChild->tabLabelWidget) {
			if(nbChild->tabLabel)
				gtk_notebook_set_tab_label_text(notebook, child->widget, nbChild->tabLabel);
			else {
				gchar tabLabel[256];
				g_snprintf(tabLabel, sizeof(tabLabel), "Page %d", i);
				gtk_notebook_set_tab_label_text(notebook, child->widget, tabLabel);
			}
		}
		if(nbChild->menuLabelWidget) {
			gtk_notebook_set_menu_label(notebook, child->widget, nbChild->menuLabelWidget);
		} else {
			if(nbChild->menuLabel)
				gtk_notebook_set_menu_label_text(notebook, child->widget, nbChild->menuLabel);
		}
		gtk_notebook_set_tab_label_packing(notebook, child->widget, GetExpand(nbChild->packOptions), GetFill(nbChild->packOptions), GTK_PACK_START);
		gtk_notebook_set_tab_detachable(notebook, child->widget, nbChild->detachable);
		gtk_notebook_set_tab_reorderable(notebook, child->widget, nbChild->reorderable);

		if(!sorted[i])
			g_object_unref(child);
		
	}

	g_slice_free1(sizeof(gpointer)*capacity, sorted);	
}

void ContainerTableSet(GtkTable * table, PtrArray * children, int capX, int capY) {
	int i;
	gtk_table_resize(table, capX, capY);
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowChild * child= CROW_CHILD(children->ptrs[i]);
			CrowTableChild * tblChild= CROW_TABLE_CHILD(children->ptrs[i]);
			gtk_table_attach(
				table, child->widget,
				child->place.a, child->place.a + child->place.c,
				child->place.b, child->place.b + child->place.d,
				GetAttachOptions(tblChild->xExpand, tblChild->xFill, tblChild->xShrink),
				GetAttachOptions(tblChild->yExpand, tblChild->yFill, tblChild->yShrink),
				tblChild->paddingX, tblChild->paddingY
			);
		}
}

==> guiloader-2.12.0.tar.bz2: guiloader-2.12.0/guiloader/containers.c <==
/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006-2008 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#include "containers.h"

// CrowChild -----------------------------------------------------------------

GType crow_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowChild), 
			0, (GInstanceInitFunc)crow_child_init,
		};
		object_type= g_type_register_static(G_TYPE_OBJECT, "CrowChild", &object_info, 0);
	}
	return object_type;
}

void crow_child_init(CrowChild * child) {
	child->widget= NULL;
	child->place.a= 0;
	child->place.b= 0;
	child->place.c= 0;
	child->place.d= 0;
}

// CrowRowChild --------------------------------------------------------------

GType crow_row_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowRowChild), 
			0, NULL,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowRowChild", &object_info, 0);
	}
	return object_type;
}

// CrowXYChild ---------------------------------------------------------------

GType crow_xy_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowXYChild), 
			0, NULL,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowXYChild", &object_info, 0);
	}
	return object_type;
}

// CrowBoxChild --------------------------------------------------------------

GType crow_box_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowBoxChild), 
			0, (GInstanceInitFunc)crow_box_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowBoxChild", &object_info, 0);
	}
	return object_type;
}

void crow_box_child_init(CrowBoxChild * child) {
	CROW_CHILD(child)->place.b= GTK_PACK_START;
	child->options= CROW_PACK_EXPAND_WIDGET;
	child->padding= 0;
}

// CrowButtonBoxChild --------------------------------------------------------

GType crow_button_box_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowButtonBoxChild), 
			0, (GInstanceInitFunc)crow_button_box_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_BOX_CHILD, "CrowButtonBoxChild", &object_info, 0);
	}
	return object_type;
}

void crow_button_box_child_init(CrowButtonBoxChild * child) {
	child->secondary= FALSE;
}

// CrowPanedChild ------------------------------------------------------------

GType crow_paned_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowPanedChild), 
			0, (GInstanceInitFunc)crow_paned_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowPanedChild", &object_info, 0);
	}
	return object_type;
}

void crow_paned_child_init(CrowPanedChild * child) {
	child->resize= TRUE;
	child->shrink= TRUE;
}

// CrowNotebookChild ---------------------------------------------------------

GType crow_notebook_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowNotebookChild), 
			0, (GInstanceInitFunc)crow_notebook_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowNotebookChild", &object_info, 0);
	}
	return object_type;
}

void crow_notebook_child_init(CrowNotebookChild * child) {
	child->tabLabel= NULL;
	child->tabLabelWidget= NULL;
	child->menuLabel= NULL;
	child->menuLabelWidget= NULL;
	child->packOptions= CROW_PACK_SHRINK;
	child->detachable= FALSE;
	child->reorderable= FALSE;
}

// CrowTableChild ------------------------------------------------------------

GType crow_table_child_get_type(void) {
	static GType object_type= 0;
	if(object_type==0) {
		static const GTypeInfo object_info= {
			sizeof(GObjectClass), 
			NULL, NULL, NULL, NULL, NULL, 
			sizeof(CrowTableChild), 
			0, (GInstanceInitFunc)crow_table_child_init,
		};
		object_type= g_type_register_static(CROW_TYPE_CHILD, "CrowTableChild", &object_info, 0);
	}
	return object_type;
}

void crow_table_child_init(CrowTableChild * child) {
	CROW_CHILD(child)->place.c= 1;
	CROW_CHILD(child)->place.d= 1;
	child->paddingX= 0;
	child->paddingY= 0;
	child->xExpand= TRUE;
	child->xFill= TRUE;
	child->xShrink= FALSE;
	child->yExpand= TRUE;
	child->yFill= TRUE;
	child->yShrink= FALSE;
}

// -----------------------------------------------------------------------------

gboolean ContainerIsEmpty(GtkContainer * container) {
	return gtk_container_get_children(container) == NULL;
}

gboolean GetExpand(CrowPackOptions packOptions) {
	return packOptions != CROW_PACK_SHRINK;
}

gboolean GetFill(CrowPackOptions packOptions) {
	return packOptions == CROW_PACK_EXPAND_WIDGET;
}

GtkAttachOptions GetAttachOptions(gboolean expand, gboolean fill, gboolean shrink) {
	GtkAttachOptions result= 0;
	if(expand)
		result |= GTK_EXPAND;
	if(fill)
		result |= GTK_FILL;
	if(shrink)
		result |= GTK_SHRINK;
	return result;
}

GtkWidget * CreatePlaceholder(void) {
	GtkWidget * widget= gtk_drawing_area_new();
	gtk_widget_set_size_request(widget, 40, 40);
	g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(ExposePlaceholder), NULL);
	gtk_widget_show(widget);
	return widget;
}

gboolean ExposePlaceholder(GtkWidget * widget, GdkEventExpose * event, gpointer data) {
	DrawGrid(widget->window, widget->style, FALSE);
	return FALSE;
}

void ContainerFixedSet(GtkFixed * fixed, PtrArray * children) {
	int i;
	for(i= 0; i < children->size; ++i) {
		CrowChild * child= CROW_CHILD(children->ptrs[i]);
		gtk_fixed_put(fixed, child->widget, child->place.a, child->place.b);
	}
}

void ContainerLayoutSet(GtkLayout * layout, PtrArray * children) {
	int i;
	for(i= 0; i < children->size; ++i) {
		CrowChild * child= CROW_CHILD(children->ptrs[i]);
		gtk_layout_put(layout, child->widget, child->place.a, child->place.b);
	}
}

void ContainerBinSet(GtkBin * bin, PtrArray * children) {
	CrowChild * child;
	Check(children && children->size==1);
	child= CROW_CHILD(children->ptrs[0]);
	gtk_container_add(GTK_CONTAINER(bin), child->widget);
}

void ContainerScrolledWindowSet(GtkScrolledWindow * scroll, PtrArray * children) {
	CrowChild * child;
	Check(children && children->size==1);
	child= CROW_CHILD(children->ptrs[0]);
	if(GTK_WIDGET_GET_CLASS(child->widget)->set_scroll_adjustments_signal != 0)
		gtk_container_add(GTK_CONTAINER(scroll), child->widget);
	else {
		GtkViewport * viewport;
		gtk_scrolled_window_add_with_viewport(scroll, child->widget);
		viewport= GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(scroll)));
		g_object_set_data(G_OBJECT(viewport), "CrowAutoViewportHint", "true");
	}
}

void ContainerDialogSet(GtkDialog * dialog, PtrArray * children) {
	CrowChild * child;
	GtkBox * box;
	Check(children && children->size==1);
	child= CROW_CHILD(children->ptrs[0]);
	box= GTK_BOX(dialog->vbox); 
	gtk_box_pack_start_defaults(box, child->widget);
}

void ContainerPanedSet(GtkPaned * paned, PtrArray * children) {
	int i;
	Check(children && children->size <= 2);
	for(i= 0; i < children->size; ++i) {
		CrowPanedChild * child= CROW_PANED_CHILD(children->ptrs[i]);
		if(CROW_CHILD(child)->place.a==0)
			gtk_paned_pack1(paned, CROW_CHILD(child)->widget, child->resize, child->shrink);
		else {
			Check(CROW_CHILD(child)->place.a==1);
			gtk_paned_pack2(paned, CROW_CHILD(child)->widget, child->resize, child->shrink);
		}
	}
}

void ContainerBoxSet(GtkBox * box, PtrArray * children, int capS, int capE) {
	PCrowChild * sorted= g_slice_alloc0(sizeof(gpointer)*(capS+capE));

	int i;
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowChild * child= CROW_CHILD(children->ptrs[i]);
			if(child->place.b == GTK_PACK_START) {
				Check(child->place.a < capS);
				Check(!sorted[child->place.a]);
				sorted[child->place.a]= child;
			} else {
				Check(child->place.b == GTK_PACK_END);
				Check(child->place.a < capE);
				Check(!sorted[capS + child->place.a]);
				sorted[capS + child->place.a]= child;
			}
		}

	for(i=0; i<capS+capE; ++i) {
		CrowChild * child= sorted[i];
		CrowBoxChild * boxChild;
		
		if(!child) {
			child= g_object_new(CROW_TYPE_BOX_CHILD, NULL);
			if(i<capS) {
				child->place.a= i;
				child->place.b= GTK_PACK_START;
			} else {
				child->place.a= i-capS;
				child->place.b= GTK_PACK_END;
			}
			child->widget= CreatePlaceholder();
		}

		boxChild= CROW_BOX_CHILD(child);
		if(child->place.b == GTK_PACK_START)
			gtk_box_pack_start(box, child->widget, GetExpand(boxChild->options), GetFill(boxChild->options), boxChild->padding);
		else
			gtk_box_pack_end(box, child->widget, GetExpand(boxChild->options), GetFill(boxChild->options), boxChild->padding);

		if(!sorted[i])
			g_object_unref(child);
		
	}
	
	g_slice_free1(sizeof(gpointer)*(capS+capE), sorted);	
}

void ContainerButtonBoxSet(GtkButtonBox * box, PtrArray * children, int capS, int capE) {
	int i;
	ContainerBoxSet(GTK_BOX(box), children, capS, capE);
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowButtonBoxChild * child= CROW_BUTTON_BOX_CHILD(children->ptrs[i]);
			if(child->secondary)
				gtk_button_box_set_child_secondary(box, CROW_CHILD(child)->widget, TRUE);
		}
}

void ContainerNotebookSet(GtkNotebook * notebook, PtrArray * children, int capacity) {
	PCrowChild * sorted= g_slice_alloc0(sizeof(gpointer)*capacity);

	int i;
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowChild * child= CROW_CHILD(children->ptrs[i]);
			Check(child->place.a < capacity);
			Check(!sorted[child->place.a]);
			sorted[child->place.a]= child;
		}

	for(i=0; i<capacity; ++i) {
		CrowChild * child= sorted[i];
		CrowNotebookChild * nbChild;
		
		if(!child) {
			child= g_object_new(CROW_TYPE_NOTEBOOK_CHILD, NULL);
			child->place.a= i;
			child->widget= CreatePlaceholder();
		}

		nbChild= CROW_NOTEBOOK_CHILD(child);
		gtk_notebook_append_page(notebook, child->widget, nbChild->tabLabelWidget);
		if(!nbChild->tabLabelWidget) {
			if(nbChild->tabLabel)
				gtk_notebook_set_tab_label_text(notebook, child->widget, nbChild->tabLabel);
			else {
				gchar tabLabel[256];
				g_snprintf(tabLabel, sizeof(tabLabel), _("Page %d"), i);
				gtk_notebook_set_tab_label_text(notebook, child->widget, tabLabel);
			}
		}
		if(nbChild->menuLabelWidget) {
			gtk_notebook_set_menu_label(notebook, child->widget, nbChild->menuLabelWidget);
		} else {
			if(nbChild->menuLabel)
				gtk_notebook_set_menu_label_text(notebook, child->widget, nbChild->menuLabel);
		}
		gtk_notebook_set_tab_label_packing(notebook, child->widget, GetExpand(nbChild->packOptions), GetFill(nbChild->packOptions), GTK_PACK_START);
		gtk_notebook_set_tab_detachable(notebook, child->widget, nbChild->detachable);
		gtk_notebook_set_tab_reorderable(notebook, child->widget, nbChild->reorderable);

		if(!sorted[i])
			g_object_unref(child);
		
	}

	g_slice_free1(sizeof(gpointer)*capacity, sorted);	
}

void ContainerTableSet(GtkTable * table, PtrArray * children, int capX, int capY) {
	int i;
	gtk_table_resize(table, capX, capY);
	if(children)
		for(i= 0; i < children->size; ++i) {
			CrowChild * child= CROW_CHILD(children->ptrs[i]);
			CrowTableChild * tblChild= CROW_TABLE_CHILD(children->ptrs[i]);
			gtk_table_attach(
				table, child->widget,
				child->place.a, child->place.a + child->place.c,
				child->place.b, child->place.b + child->place.d,
				GetAttachOptions(tblChild->xExpand, tblChild->xFill, tblChild->xShrink),
				GetAttachOptions(tblChild->yExpand, tblChild->yFill, tblChild->yShrink),
				tblChild->paddingX, tblChild->paddingY
			);
		}
}
